// Последнее обновление 2017-05-31 by Phisik
// В рамках разгрузки кода раскидываем функции по файлам
// Тут мы работаем с EEPROM

#include "configuration.h"
#include "declarations.h"



unsigned int ee_addr; // Переменная для хранения текущего адреса чтения/записи в EEPROM
unsigned char eeReadChar()
{
  unsigned char lByte = EEPROM.read(ee_addr);
  ee_addr++;
  return lByte;
}

void eeWriteChar(char p_value)
{
  if (EEPROM.read(ee_addr) != p_value)
    EEPROM.write(ee_addr, p_value);
  ee_addr++;
}

void eeWriteInt(int p_value)
{
  unsigned char lByte = (unsigned char) (p_value);
  unsigned char hByte = (unsigned char) (p_value >> 8);

  // Для экономии ресурса флеш памяти, сначала проверяем ее содержимое и запись производим только если значение отличается.
  eeWriteChar(lByte);
  eeWriteChar(hByte);
}

//This function will read a 2 byte integer from the eeprom at the specified address and address + 1
unsigned int eeReadInt()
{
  unsigned char lByte = eeReadChar();
  unsigned char hByte = eeReadChar();
  return (unsigned int) (lByte) | (unsigned int)  (hByte << 8);
}

void writeEEPROM()
{
  int i;
  flAutoDetectPowerTEN = 0;
  ee_addr = 0;
  eeWriteChar(PR_REWRITE_EEPROM);
  eeWriteChar(IspReg);
  eeWriteInt(Power);
  eeWriteInt(UstPower);
  eeWriteInt(UstPowerReg);
  eeWriteInt(TempTerm);
  eeWriteInt(Delta);
  eeWriteChar(FlToUSART);
  eeWriteInt(Temp1P);
  eeWriteInt(Temp2P);
  eeWriteInt(Temp3P);
  for (i = 0; i < MAX_DS1820; i++)
    eeWriteChar(ds1820_popr[i]);

  eeWriteInt(PowerGlvDistil);
  eeWriteInt(PowerDistil);
  eeWriteInt(TempDeflBegDistil);
  eeWriteInt(TempDefl);
  eeWriteChar(DeltaDefl);

  eeWriteChar(BeepEndProcess);
  eeWriteChar(BeepStateProcess);
  eeWriteChar(BeepKeyPress);

  eeWriteInt(tEndDistDefl);
  eeWriteInt(PowerVarkaZerno);
  eeWriteInt(PowerMinute);
  eeWriteInt(NaprPeregrev);
  eeWriteChar(DecrementCHIM);
  eeWriteChar(IncrementCHIM);
  eeWriteInt(TimeAutoIncCHIM);
  eeWriteChar(CntCHIM);
  for (i = 0; i < COUNT_CHIM; i++)
  {
    eeWriteInt(tempK[i]);
    eeWriteChar(CHIM[i]);
  }
  eeWriteChar(CorrectASC712);
  eeWriteChar(PIDTemp[0]);
  eeWriteChar(PIDTemp[1]);
  eeWriteChar(PIDTemp[2]);

  eeWriteChar(CountFractionDist);
  for (i = 0; i < MAX_CNT_FRACTION_DIST; i++)
  {
    eeWriteInt(TempFractionDist[i]);
    eeWriteInt(PowerFractionDist[i]);
  }

  // Phisik: поддержка сортировки сенсоров
#if ENABLE_SENSOR_SORTING
  for (i = 0; i < MAX_DS1820; i++) {
    eeWriteChar(ds1820_nums[i]);
  }
#endif
}

void readEEPROM()
{
  int i;
  unsigned char ctmp;
  ee_addr = 0;
  eeReadChar();
  IspReg = eeReadChar();
  Power = eeReadInt();
  UstPower = eeReadInt();  
  UstPowerReg = eeReadInt();
  TempTerm = eeReadInt();
  Delta = eeReadInt();
  FlToUSART = eeReadChar();
  Temp1P = eeReadInt();
  Temp2P = eeReadInt();
  Temp3P = eeReadInt();
  for (i = 0; i < MAX_DS1820; i++)
    ds1820_popr[i] = eeReadChar();

  PowerGlvDistil = eeReadInt();
  PowerDistil = eeReadInt();

  TempDeflBegDistil = eeReadInt();
  TempDefl = eeReadInt();
  DeltaDefl = eeReadChar();

  BeepEndProcess = eeReadChar();
  BeepStateProcess = eeReadChar();
  BeepKeyPress = eeReadChar();

  tEndDistDefl = eeReadInt();
  PowerVarkaZerno = eeReadInt();
  PowerMinute = eeReadInt();
  if (PowerMinute > 10000 || PowerMinute < 0) PowerMinute = Power;
  NaprPeregrev = eeReadInt();
  if (NaprPeregrev > 1000 ) NaprPeregrev = 300;

  DecrementCHIM = eeReadChar();
  if (DecrementCHIM == -1) DecrementCHIM = 10;
  IncrementCHIM = eeReadChar();
  if (IncrementCHIM == -1) IncrementCHIM = 0;
  TimeAutoIncCHIM = eeReadInt();
  if (TimeAutoIncCHIM == -1) TimeAutoIncCHIM = 600;

  CntCHIM = eeReadChar();
  for (i = 0; i < COUNT_CHIM; i++)
  {
    tempK[i] = eeReadInt();
    CHIM[i] = eeReadChar();
  }
  CorrectASC712 = eeReadChar();
  if (CorrectASC712 == -1) CorrectASC712 = 0;

  PIDTemp[0] = eeReadChar();
  PIDTemp[1] = eeReadChar();
  PIDTemp[2] = eeReadChar();
  if (PIDTemp[0] == -1)
  {
    PIDTemp[0] = 20;
    PIDTemp[1] = 10;
    PIDTemp[2] = 30;
  }

  CountFractionDist = eeReadChar();
  for (i = 0; i < MAX_CNT_FRACTION_DIST; i++)
  {
    TempFractionDist[i] = eeReadInt();
    if (TempFractionDist[i] == -1)  TempFractionDist[i] = 0;
    PowerFractionDist[i] = eeReadInt();
    if (PowerFractionDist[i] == -1)  PowerFractionDist[i] = 0;
  }

  PIDTime == eeReadChar();
  if (PIDTime == 255) PIDTime = 15;

  // Phisik: поддержка сортировки сенсоров
#if ENABLE_SENSOR_SORTING
  for (i = 0; i < MAX_DS1820; i++) {
    ds1820_nums[i] = eeReadChar();
    ds1820_nums[i] = min(ds1820_nums[i], MAX_DS1820 - 1);
  }
#endif

}
